@@ -103,8 +103,13 @@ librdf_init_storage(librdf_world *world)
 #ifdef MODULAR_LIBRDF
 
   if (!world->storage_modules)
+#ifdef RAPTOR_V2_AVAILABLE
+    world->storage_modules = raptor_new_sequence(
+        (raptor_data_free_handler)lt_dlclose, NULL);
+#else
     world->storage_modules = raptor_new_sequence(
         (raptor_sequence_free_handler*)lt_dlclose, NULL);
+#endif
 
   librdf_storage_load_all_modules(world);
 
@@ -339,7 +344,11 @@ librdf_storage_register_factory(librdf_world* world,
 #endif
 
   if(!world->storages) {
-    world->storages=raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_storage_factory, NULL);
+#ifdef RAPTOR_V2_AVAILABLE
+    world->storages = raptor_new_sequence((raptor_data_free_handler)librdf_free_storage_factory, NULL);
+#else
+    world->storages = raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_storage_factory, NULL);
+#endif
     if(!world->storages)
       goto failed;
   }
@@ -558,9 +567,11 @@ librdf_new_storage_with_options(librdf_world *world,
 
 
 /**
- * librdf_new_storage_from_storage - Copy constructor - create a new librdf_storage object from an existing one
+ * librdf_new_storage_from_storage:
  * @old_storage: the existing storage #librdf_storage to use
  *
+ * Copy constructor - create a new librdf_storage object from an existing one
+ *
  * Should create a new storage in the same context as the existing one
  * as appropriate for the storage.  For example, in a RDBMS storage
  * it would be a new database, or in on disk it would be a new
@@ -681,8 +692,9 @@ librdf_new_storage_from_factory(librdf_world *world,
 void
 librdf_free_storage(librdf_storage* storage) 
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(storage, librdf_storage);
-
+  if(!storage)
+    return;
+  
   if(--storage->usage)
     return;
 
@@ -698,6 +710,7 @@ librdf_free_storage(librdf_storage* storage)
  * @storage: #librdf_storage object
  *
  * Increment storage reference count by one.
+ *
  * This function is intended to be internal to librdf storage modules.
  **/
 void
@@ -708,11 +721,11 @@ librdf_storage_add_reference(librdf_storage *storage)
 
 
 /**
- * librdf_storage_remove_reference(libdf_storage *storage)
+ * librdf_storage_remove_reference:
  * @storage: #librdf_storage object
  *
- * Decrement storage reference count by one.
- * Free the storage if reference count becomes zero.
+ * Decrement storage reference count by one and free the storage if reference count becomes zero.
+ *
  * This function is intended to be internal to librdf storage modules.
  **/
 void
@@ -1116,7 +1129,7 @@ librdf_storage_stream_to_node_iterator_get_method(void* iterator, int flags)
 
           case LIBRDF_STATEMENT_ALL:
             default: /* error */
-              librdf_log(statement->world,
+              librdf_log(context->storage->world,
                          0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                          "Unknown statement part %d", context->want);
               node=NULL;
@@ -1128,14 +1141,14 @@ librdf_storage_stream_to_node_iterator_get_method(void* iterator, int flags)
       
     case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
       if(!context->context_node) {
-        node=(librdf_node*)librdf_stream_get_context(context->stream);
+        node = librdf_stream_get_context2(context->stream);
         context->context_node=node ? librdf_new_node_from_node(node) : NULL;
       }
       node=context->context_node;
       break;
       
     default:
-      librdf_log(statement->world,
+      librdf_log(context->storage->world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "Unknown iterator method flag %d", flags);
       node=NULL;
@@ -1171,12 +1184,14 @@ librdf_storage_stream_to_node_iterator_finished(void* iterator)
 
 
 /*
- * librdf_storage_node_stream_to_node_create - Create a stream for get sources, targets or arcs methods using find_statements method
+ * librdf_storage_node_stream_to_node_create:
  * @storage: the storage object to use
  * @node1: the first node to encode in the key (or NULL if not needed)
  * @node2: the second node to encode in the key (or NULL if not needed)
  * @want: the field required from the statement
  *
+ * INTERNAL - Create a stream for get sources, targets or arcs methods using find_statements method
+ *
  * node1 and node2 cannot both be NULL
  * 
  * Return value: a new #librdf_iterator or NULL on failure
@@ -1560,7 +1575,7 @@ librdf_storage_context_remove_statement(librdf_storage* storage,
                                         librdf_statement* statement) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 1);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(context, librdf_statement, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, 1);
 
   if(!storage->factory->context_remove_statement)
     return 1;
@@ -1665,7 +1680,10 @@ librdf_storage_supports_query(librdf_storage* storage, librdf_query *query)
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 0);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, librdf_query, 0);
 
-  return 0;
+  if(storage->factory->supports_query)
+    return storage->factory->supports_query(storage, query);
+  else
+    return 0;
 }
 
 
@@ -1684,7 +1702,10 @@ librdf_storage_query_execute(librdf_storage* storage, librdf_query *query)
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, librdf_query, NULL);
 
-  return NULL;
+  if(storage->factory->supports_query)
+    return storage->factory->query_execute(storage, query);
+  else
+    return NULL;
 }
 
 
@@ -1967,7 +1988,11 @@ main(int argc, char *argv[])
   /* triples of arguments to librdf_new_storage */
   const char* const storages[] = {
 	"memory", NULL, "contexts='yes'",
+#ifdef HAVE_BDB_HASH
 	"hashes", "test", "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'",
+#else
+	"hashes", "test", "hash-type='memory',write='yes',new='yes',contexts='yes'",
+#endif
     #ifdef STORAGE_TREES
 	    "trees", "test", "contexts='yes'",
     #endif
