@@ -590,6 +590,7 @@ librdf_storage_hashes_add_remove_statement(librdf_storage* storage,
   librdf_storage_hashes_instance* context=(librdf_storage_hashes_instance*)storage->instance;
   int i;
   int status=0;
+  librdf_world* world = storage->world;
 
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   if(is_addition)
@@ -610,7 +611,7 @@ librdf_storage_hashes_add_remove_statement(librdf_storage* storage,
     if(!fields)
       continue;
     
-    key_len=librdf_statement_encode_parts(statement, NULL, NULL, 0, fields);
+    key_len=librdf_statement_encode_parts2(world, statement, NULL, NULL, 0, fields);
     if(!key_len)
       return 1;
     if(librdf_storage_hashes_grow_buffer(&context->key_buffer, 
@@ -619,9 +620,9 @@ librdf_storage_hashes_add_remove_statement(librdf_storage* storage,
       break;
     }
        
-    if(!librdf_statement_encode_parts(statement, NULL,
-                                      context->key_buffer,
-                                      context->key_buffer_len, fields)) {
+    if(!librdf_statement_encode_parts2(world, statement, NULL,
+                                       context->key_buffer,
+                                       context->key_buffer_len, fields)) {
       status=1;
       break;
     }
@@ -633,8 +634,8 @@ librdf_storage_hashes_add_remove_statement(librdf_storage* storage,
     if(!fields)
       continue;
     
-    value_len=librdf_statement_encode_parts(statement, context_node,
-                                            NULL, 0, fields);
+    value_len=librdf_statement_encode_parts2(world, statement, context_node,
+                                             NULL, 0, fields);
     if(!value_len) {
       status=1;
       break;
@@ -646,9 +647,9 @@ librdf_storage_hashes_add_remove_statement(librdf_storage* storage,
       break;
     }
        
-    if(!librdf_statement_encode_parts(statement, context_node,
-                                      context->value_buffer,
-                                      context->value_buffer_len, fields)) {
+    if(!librdf_statement_encode_parts2(world, statement, context_node,
+                                       context->value_buffer,
+                                       context->value_buffer_len, fields)) {
       status=1;
       break;
     }
@@ -727,6 +728,7 @@ librdf_storage_hashes_contains_statement(librdf_storage* storage, librdf_stateme
   int hash_index=context->all_statements_hash_index;
   librdf_statement_part fields;
   int status;
+  librdf_world* world = storage->world;
   
   if(storage->index_contexts) {
     /* When we have contexts, we have to use find_statements for contains
@@ -746,23 +748,23 @@ librdf_storage_hashes_contains_statement(librdf_storage* storage, librdf_stateme
 
   /* ENCODE KEY */
   fields=(librdf_statement_part)context->hash_descriptions[hash_index]->key_fields;
-  key_len=librdf_statement_encode_parts(statement, NULL,
-                                        NULL, 0, fields);
+  key_len=librdf_statement_encode_parts2(world, statement, NULL,
+                                         NULL, 0, fields);
   if(!key_len)
     return 1;
   if(!(key_buffer=(unsigned char*)LIBRDF_MALLOC(data, key_len)))
     return 1;
        
-  if(!librdf_statement_encode_parts(statement, NULL,
-                                    key_buffer, key_len, fields)) {
+  if(!librdf_statement_encode_parts2(world, statement, NULL,
+                                     key_buffer, key_len, fields)) {
     LIBRDF_FREE(data, key_buffer);
     return 1;
   }
 
   /* ENCODE VALUE */
   fields=(librdf_statement_part)context->hash_descriptions[hash_index]->value_fields;
-  value_len=librdf_statement_encode_parts(statement, NULL,
-                                          NULL, 0, fields);
+  value_len=librdf_statement_encode_parts2(world, statement, NULL,
+                                           NULL, 0, fields);
   if(!value_len) {
     LIBRDF_FREE(data, key_buffer);
     return 1;
@@ -774,8 +776,8 @@ librdf_storage_hashes_contains_statement(librdf_storage* storage, librdf_stateme
   }
 
        
-  if(!librdf_statement_encode_parts(statement, NULL,
-                                    value_buffer, value_len, fields)) {
+  if(!librdf_statement_encode_parts2(world, statement, NULL,
+                                     value_buffer, value_len, fields)) {
     LIBRDF_FREE(data, key_buffer);
     LIBRDF_FREE(data, value_buffer);
     return 1;
@@ -916,7 +918,10 @@ librdf_storage_hashes_serialise_get_statement(void* context, int flags)
   librdf_storage_hashes_serialise_stream_context* scontext=(librdf_storage_hashes_serialise_stream_context*)context;
   librdf_hash_datum* hd;
   librdf_node** cnp=NULL;
-
+  librdf_world* world;
+  
+  world = scontext->storage->world;
+  
   if(scontext->search_node) {
     switch(flags) {
       case LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT:
@@ -955,16 +960,16 @@ librdf_storage_hashes_serialise_get_statement(void* context, int flags)
       hd=(librdf_hash_datum*)librdf_iterator_get_key(scontext->iterator);
       
       /* decode key content */
-      if(!librdf_statement_decode_parts(&scontext->current, NULL,
-                                        (unsigned char*)hd->data, hd->size)) {
+      if(!librdf_statement_decode2(world, &scontext->current, NULL,
+                                   (unsigned char*)hd->data, hd->size)) {
         return NULL;
       }
       
       hd=(librdf_hash_datum*)librdf_iterator_get_value(scontext->iterator);
       
       /* decode value content and optional context */
-      if(!librdf_statement_decode_parts(&scontext->current, cnp,
-                                        (unsigned char*)hd->data, hd->size)) {
+      if(!librdf_statement_decode2(world, &scontext->current, cnp,
+                                   (unsigned char*)hd->data, hd->size)) {
         return NULL;
       }
 
@@ -1100,6 +1105,9 @@ librdf_storage_hashes_node_iterator_get_method(void* iterator, int flags)
   librdf_storage_hashes_node_iterator_context* context=(librdf_storage_hashes_node_iterator_context*)iterator;
   librdf_node* node;
   librdf_hash_datum* value;
+  librdf_world* world;
+  
+  world = context->storage->world;
   
   if(librdf_iterator_end(context->iterator))
     return NULL;
@@ -1116,9 +1124,9 @@ librdf_storage_hashes_node_iterator_get_method(void* iterator, int flags)
     context->context_node=NULL;
       
     /* decode value content and optional context */
-    if(!librdf_statement_decode_parts(&context->statement,
-                                      &context->context_node,
-                                      (unsigned char*)value->data, value->size))
+    if(!librdf_statement_decode2(world, &context->statement,
+                                 &context->context_node,
+                                 (unsigned char*)value->data, value->size))
       return NULL;
     librdf_statement_clear(&context->statement);
     
@@ -1170,9 +1178,9 @@ librdf_storage_hashes_node_iterator_get_method(void* iterator, int flags)
   if(!value)
     return NULL;
 
-  if(!librdf_statement_decode(&context->statement, 
-                              (unsigned char*)value->data,
-                              value->size))
+  if(!librdf_statement_decode2(world, &context->statement, NULL,
+                               (unsigned char*)value->data,
+                               value->size))
     return NULL;
 
   switch(context->want) {
@@ -1259,6 +1267,7 @@ librdf_storage_hashes_node_iterator_create(librdf_storage* storage,
   librdf_statement_part fields;
   unsigned char *key_buffer;
   librdf_iterator* iterator;
+  librdf_world* world = storage->world;
   
   icontext=(librdf_storage_hashes_node_iterator_context*)LIBRDF_CALLOC(librdf_storage_hashes_node_iterator_context, 1, sizeof(librdf_storage_hashes_node_iterator_context));
   if(!icontext)
@@ -1323,8 +1332,8 @@ librdf_storage_hashes_node_iterator_create(librdf_storage* storage,
 
   /* ENCODE KEY */
   fields=(librdf_statement_part)scontext->hash_descriptions[hash_index]->key_fields;
-  icontext->key.size=librdf_statement_encode_parts(&icontext->statement, NULL,
-                                                   NULL, 0, fields);
+  icontext->key.size=librdf_statement_encode_parts2(world, &icontext->statement, NULL,
+                                                    NULL, 0, fields);
   if(!icontext->key.size) {
     LIBRDF_FREE(librdf_storage_hashes_node_iterator_context, icontext);
     return NULL;
@@ -1339,8 +1348,8 @@ librdf_storage_hashes_node_iterator_create(librdf_storage* storage,
    */
   librdf_storage_add_reference(icontext->storage);
 
-  if(!librdf_statement_encode_parts(&icontext->statement, NULL,
-                                    key_buffer, icontext->key.size, fields)) {
+  if(!librdf_statement_encode_parts2(world, &icontext->statement, NULL,
+                                     key_buffer, icontext->key.size, fields)) {
     LIBRDF_FREE(data, key_buffer);
     librdf_storage_hashes_node_iterator_finished(icontext);
     return NULL;
@@ -1420,6 +1429,7 @@ librdf_storage_hashes_context_add_statement(librdf_storage* storage,
   librdf_hash_datum key, value; /* on stack - not allocated */
   int size;
   int status;
+  librdf_world* world = storage->world;
   
   if(context->contexts_index <0) {
     librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
@@ -1436,10 +1446,10 @@ librdf_storage_hashes_context_add_statement(librdf_storage* storage,
   key.size=librdf_node_encode(context_node, 
                                (unsigned char*)key.data, size);
 
-  size=librdf_statement_encode(statement, NULL, 0);
+  size=librdf_statement_encode2(world, statement, NULL, 0);
 
   value.data=(char*)LIBRDF_MALLOC(cstring, size);
-  value.size=librdf_statement_encode(statement, (unsigned char*)value.data, size);
+  value.size=librdf_statement_encode2(world, statement, (unsigned char*)value.data, size);
 
   status=librdf_hash_put(context->hashes[context->contexts_index], &key, &value);
   LIBRDF_FREE(data, key.data);
@@ -1468,6 +1478,7 @@ librdf_storage_hashes_context_remove_statement(librdf_storage* storage,
   librdf_hash_datum key, value; /* on stack - not allocated */
   int size;
   int status;
+  librdf_world* world = storage->world;
   
   if(context_node && context->contexts_index <0) {
     librdf_log(storage->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_STORAGE, NULL,
@@ -1483,10 +1494,10 @@ librdf_storage_hashes_context_remove_statement(librdf_storage* storage,
   key.size=librdf_node_encode(context_node,
                                (unsigned char*)key.data, size);
 
-  size=librdf_statement_encode(statement, NULL, 0);
+  size=librdf_statement_encode2(world, statement, NULL, 0);
 
   value.data=(char*)LIBRDF_MALLOC(cstring, size);
-  value.size=librdf_statement_encode(statement, (unsigned char*)value.data, size);
+  value.size=librdf_statement_encode2(world, statement, (unsigned char*)value.data, size);
 
   status=librdf_hash_delete(context->hashes[context->contexts_index], &key, &value);
   LIBRDF_FREE(data, key.data);
@@ -1605,42 +1616,37 @@ librdf_storage_hashes_context_serialise_next_statement(void* context)
 static void*
 librdf_storage_hashes_context_serialise_get_statement(void* context, int flags)
 {
-  librdf_storage_hashes_context_serialise_stream_context* scontext=(librdf_storage_hashes_context_serialise_stream_context*)context;
+  librdf_storage_hashes_context_serialise_stream_context* scontext;
   librdf_hash_datum* v;
-  librdf_node** cnp=NULL;
+  librdf_world* world;
+
+  scontext = (librdf_storage_hashes_context_serialise_stream_context*)context;
+  world = scontext->storage->world;
 
   switch(flags) {
     case LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT:
     case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
 
       if(scontext->current_is_ok) {
-        if(flags==LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT)
+        if(flags == LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT)
           return &scontext->current;
         else
           return scontext->context_node;
       }
 
-      /* current stuff is out of date - get new cached answers */
-      if(scontext->index_contexts) {
-        if(scontext->context_node)
-          librdf_free_node(scontext->context_node);
-        scontext->context_node=NULL;
-        cnp=&scontext->context_node;
-      }
-      
       librdf_statement_clear(&scontext->current);
 
-      v=(librdf_hash_datum*)librdf_iterator_get_value(scontext->iterator);
+      v = (librdf_hash_datum*)librdf_iterator_get_value(scontext->iterator);
       
       /* decode value content and optional context */
-      if(!librdf_statement_decode_parts(&scontext->current, cnp,
-                                        (unsigned char*)v->data, v->size)) {
+      if(!librdf_statement_decode2(world, &scontext->current, NULL,
+                                   (unsigned char*)v->data, v->size)) {
         return NULL;
       }
       
-      scontext->current_is_ok=1;
+      scontext->current_is_ok = 1;
 
-      if(flags==LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT)
+      if(flags == LIBRDF_ITERATOR_GET_METHOD_GET_OBJECT)
         return &scontext->current;
       else
         return scontext->context_node;
