@@ -59,6 +59,7 @@
 void
 librdf_init_uri(librdf_world *world)
 {
+#ifndef LIBRDF_USE_RAPTOR_URI
   /* If no default given, create an in memory hash */
   if(!world->uris_hash) {
     world->uris_hash=librdf_new_hash(world, NULL);
@@ -71,6 +72,7 @@ librdf_init_uri(librdf_world *world)
     if(librdf_hash_open(world->uris_hash, NULL, 0, 1, 1, NULL))
       LIBRDF_FATAL1(world, LIBRDF_FROM_URI, "Failed to open URI hash");
   }
+#endif
 }
 
 
@@ -85,16 +87,129 @@ librdf_init_uri(librdf_world *world)
 void
 librdf_finish_uri(librdf_world *world)
 {
+#ifndef LIBRDF_USE_RAPTOR_URI
   if (world->uris_hash) {
     librdf_hash_close(world->uris_hash);
 
     if(world->uris_hash_allocated_here)
       librdf_free_hash(world->uris_hash);
   }
+#endif
 }
 
 
 
+/**
+ * librdf_new_uri2:
+ * @world: redland world object
+ * @uri_string: URI in string form
+ * @length: length of string
+ *
+ * Constructor - create a new #librdf_uri object from a counted URI string.
+ * 
+ * A new URI is constructed from a copy of the string.  If the string
+ * is a NULL pointer or 0 length or empty (first byte is 0) then the
+ * result is NULL.
+ *
+ * Return value: a new #librdf_uri object or NULL on failure
+ **/
+librdf_uri*
+librdf_new_uri2(librdf_world *world, 
+                const unsigned char *uri_string,
+                size_t length)
+{
+#ifdef LIBRDF_USE_RAPTOR_URI
+  return raptor_new_uri_from_counted_string(world->raptor_world_ptr,
+                                            uri_string, length);
+#else
+  librdf_uri* new_uri;
+  unsigned char *new_string;
+  librdf_hash_datum key, value; /* on stack - not allocated */
+  librdf_hash_datum *old_value;
+
+  /* just to be safe */
+  memset(&key, 0, sizeof(key));
+  memset(&value, 0, sizeof(value));
+
+  librdf_world_open(world);
+
+  if(!uri_string || !length || !*uri_string)
+    return NULL;
+
+#ifdef WITH_THREADS
+  pthread_mutex_lock(world->mutex);
+#endif
+  
+  key.data = (char*)uri_string;
+  key.size = length;
+
+  /* if existing URI found in hash, return it */
+  if((old_value = librdf_hash_get_one(world->uris_hash, &key))) {
+    new_uri = *(librdf_uri**)old_value->data;
+
+#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
+    LIBRDF_DEBUG3("Found existing URI %s in hash with current usage %d\n", uri_string, new_uri->usage);
+#endif
+
+    librdf_free_hash_datum(old_value);
+    new_uri->usage++;
+
+#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
+    if(new_uri->usage > new_uri->max_usage)
+      new_uri->max_usage = new_uri->usage;
+#endif    
+
+    goto unlock;
+  }
+  
+
+  /* otherwise create a new one */
+
+#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
+  LIBRDF_DEBUG2("Creating new URI %s in hash\n", uri_string);
+#endif
+
+  new_uri = (librdf_uri*)LIBRDF_CALLOC(librdf_uri, 1, sizeof(librdf_uri));
+  if(!new_uri)
+    goto unlock;
+
+  new_uri->world = world;
+  new_uri->string_length = length;
+
+  new_string = (unsigned char*)LIBRDF_MALLOC(cstring, length+1);
+  if(!new_string) {
+    LIBRDF_FREE(librdf_uri, new_uri);
+    new_uri = NULL;
+    goto unlock;
+  }
+  
+  strcpy((char*)new_string, (const char*)uri_string);
+  new_uri->string = new_string;
+
+  new_uri->usage = 1;
+#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
+  new_uri->max_usage = 1;
+#endif
+
+  value.data = &new_uri; value.size = sizeof(librdf_uri*);
+  
+  /* store in hash: URI-string => (librdf_uri*) */
+  if(librdf_hash_put(world->uris_hash, &key, &value)) {
+    LIBRDF_FREE(cstring, new_string);
+    LIBRDF_FREE(librdf_uri, new_uri);
+    new_uri = NULL;
+  }
+
+ unlock:
+#ifdef WITH_THREADS
+  pthread_mutex_unlock(world->mutex);
+#endif
+
+  return new_uri;
+#endif /* !LIBRDF_USE_RAPTOR_URI */
+}
+
+
 /**
  * librdf_new_uri:
  * @world: redland world object
@@ -111,93 +226,12 @@ librdf_uri*
 librdf_new_uri(librdf_world *world, 
                const unsigned char *uri_string)
 {
-  librdf_uri* new_uri;
-  unsigned char *new_string;
-  int length;
-  librdf_hash_datum key, value; /* on stack - not allocated */
-  librdf_hash_datum *old_value;
-
-  /* just to be safe */
-  memset(&key, 0, sizeof(key));
-  memset(&value, 0, sizeof(value));
-
   librdf_world_open(world);
 
   if(!uri_string || !*uri_string)
     return NULL;
 
-#ifdef WITH_THREADS
-  pthread_mutex_lock(world->mutex);
-#endif
-  
-  length=strlen((const char*)uri_string);
-
-  key.data=(char*)uri_string;
-  key.size=length;
-
-  /* if existing URI found in hash, return it */
-  if((old_value=librdf_hash_get_one(world->uris_hash, &key))) {
-    new_uri=*(librdf_uri**)old_value->data;
-
-#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
-    LIBRDF_DEBUG3("Found existing URI %s in hash with current usage %d\n", uri_string, new_uri->usage);
-#endif
-
-    librdf_free_hash_datum(old_value);
-    new_uri->usage++;
-
-#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
-    if(new_uri->usage > new_uri->max_usage)
-      new_uri->max_usage=new_uri->usage;
-#endif    
-
-    goto unlock;
-  }
-  
-
-  /* otherwise create a new one */
-
-#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
-  LIBRDF_DEBUG2("Creating new URI %s in hash\n", uri_string);
-#endif
-
-  new_uri = (librdf_uri*)LIBRDF_CALLOC(librdf_uri, 1, sizeof(librdf_uri));
-  if(!new_uri)
-    goto unlock;
-
-  new_uri->world=world;
-  new_uri->string_length=length;
-
-  new_string=(unsigned char*)LIBRDF_MALLOC(cstring, length+1);
-  if(!new_string) {
-    LIBRDF_FREE(librdf_uri, new_uri);
-    new_uri=NULL;
-    goto unlock;
-  }
-  
-  strcpy((char*)new_string, (const char*)uri_string);
-  new_uri->string=new_string;
-
-  new_uri->usage=1;
-#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
-  new_uri->max_usage=1;
-#endif
-
-  value.data=&new_uri; value.size=sizeof(librdf_uri*);
-  
-  /* store in hash: URI-string => (librdf_uri*) */
-  if(librdf_hash_put(world->uris_hash, &key, &value)) {
-    LIBRDF_FREE(cstring, new_string);
-    LIBRDF_FREE(librdf_uri, new_uri);
-    new_uri=NULL;
-  }
-
- unlock:
-#ifdef WITH_THREADS
-  pthread_mutex_unlock(world->mutex);
-#endif
-
-  return new_uri;
+  return librdf_new_uri2(world, uri_string, strlen((const char*)uri_string));
 }
 
 
@@ -213,9 +247,13 @@ librdf_uri*
 librdf_new_uri_from_uri (librdf_uri* old_uri) {
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(old_uri, librdf_uri, NULL);
-  
+
+#ifdef LIBRDF_USE_RAPTOR_URI
+  return raptor_uri_copy(old_uri);
+#else
   old_uri->usage++;
   return old_uri;
+#endif
 }
 
 
@@ -231,6 +269,9 @@ librdf_new_uri_from_uri (librdf_uri* old_uri) {
 librdf_uri*
 librdf_new_uri_from_uri_local_name (librdf_uri* old_uri, 
                                     const unsigned char *local_name) {
+#ifdef LIBRDF_USE_RAPTOR_URI
+  return raptor_new_uri_from_uri_local_name(raptor_uri_get_world(old_uri), old_uri, local_name);
+#else
   int len;
   unsigned char *new_string;
   librdf_uri* new_uri;
@@ -253,6 +294,7 @@ librdf_new_uri_from_uri_local_name (librdf_uri* old_uri,
   LIBRDF_FREE(cstring, new_string);
 
   return new_uri;
+#endif /* !LIBRDF_USE_RAPTOR_URI */
 }
 
 
@@ -275,7 +317,15 @@ librdf_new_uri_normalised_to_base(const unsigned char *uri_string,
   int len;
   unsigned char *new_uri_string;
   librdf_uri *new_uri;
-  librdf_world *world=source_uri->world;
+#ifdef LIBRDF_USE_RAPTOR_URI
+#else
+  librdf_world *world = source_uri->world;
+#endif
+  unsigned char* source_uri_string;
+  size_t source_uri_string_length;
+  unsigned char* base_uri_string;
+  size_t base_uri_string_length;
+
                                     
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(source_uri, librdf_uri, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(base_uri, librdf_uri, NULL);
@@ -284,13 +334,30 @@ librdf_new_uri_normalised_to_base(const unsigned char *uri_string,
     return NULL;
 
   /* empty URI - easy, just make from base_uri */
-  if(!*uri_string && base_uri)
+  if(!*uri_string && base_uri) {
+#ifdef LIBRDF_USE_RAPTOR_URI
+    return raptor_uri_copy(base_uri);
+#else
     return librdf_new_uri_from_uri(base_uri);
+#endif /* !LIBRDF_USE_RAPTOR_URI */
+  }
+  
+  source_uri_string = librdf_uri_as_counted_string(source_uri,
+                                                   &source_uri_string_length);
+  base_uri_string = librdf_uri_as_counted_string(base_uri,
+                                                 &base_uri_string_length);
 
   /* not a fragment, and no match - easy */
   if(*uri_string != '#' &&
-     strncmp((const char*)uri_string, (const char*)source_uri->string, source_uri->string_length))
+     strncmp((const char*)uri_string, (const char*)source_uri_string,
+             source_uri_string_length)) {
+#ifdef LIBRDF_USE_RAPTOR_URI
+    raptor_world* rworld = raptor_uri_get_world(base_uri);
+    return raptor_new_uri(rworld, uri_string);
+#else
     return librdf_new_uri(world, uri_string);
+#endif
+  }
 
   /* darn - is a fragment or matches, is a prefix of the source URI */
 
@@ -299,22 +366,28 @@ librdf_new_uri_normalised_to_base(const unsigned char *uri_string,
    * be appended
    */
   if(*uri_string != '#')
-    uri_string += source_uri->string_length;
+    uri_string += source_uri_string_length;
 
   /* size of remaining bytes to copy from uri_string */
-  uri_string_len=strlen((const char*)uri_string);
+  uri_string_len = strlen((const char*)uri_string);
 
   /* total bytes */
-  len=uri_string_len + 1 + base_uri->string_length;
+  len = uri_string_len + 1 + base_uri_string_length;
 
   new_uri_string=(unsigned char*)LIBRDF_MALLOC(cstring, len);
   if(!new_uri_string)
     return NULL;
-  strncpy((char*)new_uri_string, (const char*)base_uri->string, base_uri->string_length);
+  strncpy((char*)new_uri_string, (const char*)base_uri_string,
+          base_uri_string_length);
   /* strcpy not strncpy since I want a \0 on the end */
-  strcpy((char*)new_uri_string + base_uri->string_length, (const char*)uri_string);
+  strcpy((char*)new_uri_string + base_uri_string_length,
+         (const char*)uri_string);
   
+#ifdef LIBRDF_USE_RAPTOR_URI
+  new_uri = raptor_new_uri(raptor_uri_get_world(source_uri), new_uri_string);
+#else
   new_uri=librdf_new_uri(world, new_uri_string);
+#endif /* !LIBRDF_USE_RAPTOR_URI */
   LIBRDF_FREE(cstring, new_uri_string); /* always free this even on failure */
 
   return new_uri; /* new URI or NULL from librdf_new_uri failure */
@@ -337,6 +410,11 @@ librdf_new_uri_normalised_to_base(const unsigned char *uri_string,
 librdf_uri*
 librdf_new_uri_relative_to_base(librdf_uri* base_uri,
                                 const unsigned char *uri_string) {
+#ifdef LIBRDF_USE_RAPTOR_URI
+  return raptor_new_uri_relative_to_base(raptor_uri_get_world(base_uri),
+                                         base_uri,
+                                         uri_string);
+#else
   unsigned char *buffer;
   int buffer_length;
   librdf_uri* new_uri;
@@ -363,6 +441,7 @@ librdf_new_uri_relative_to_base(librdf_uri* base_uri,
   new_uri=librdf_new_uri(world, buffer);
   LIBRDF_FREE(cstring, buffer);
   return new_uri;
+#endif /* !LIBRDF_USE_RAPTOR_URI */
 }
 
 
@@ -404,15 +483,22 @@ librdf_new_uri_from_filename(librdf_world* world, const char *filename) {
  * 
  **/
 void
-librdf_free_uri (librdf_uri* uri) 
+librdf_free_uri(librdf_uri* uri) 
 {
+#ifdef LIBRDF_USE_RAPTOR_URI
+  if(!uri)
+    return;
+  
+  raptor_free_uri(uri);
+#else
   librdf_hash_datum key; /* on stack */
 #ifdef WITH_THREADS
   librdf_world *world;
 #endif
 
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(uri, librdf_uri);
-
+  if(!uri)
+    return;
+  
 #ifdef WITH_THREADS
   world = uri->world;
   pthread_mutex_lock(world->mutex);
@@ -449,6 +535,7 @@ librdf_free_uri (librdf_uri* uri)
 #ifdef WITH_THREADS
   pthread_mutex_unlock(world->mutex);
 #endif
+#endif /* !LIBRDF_USE_RAPTOR_URI */
 }
 
 
@@ -467,8 +554,11 @@ unsigned char*
 librdf_uri_as_string (librdf_uri *uri) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, librdf_uri, NULL);
-
+#ifdef LIBRDF_USE_RAPTOR_URI
+  return raptor_uri_as_string(uri);
+#else
   return uri->string;
+#endif
 }
 
 
@@ -488,15 +578,19 @@ unsigned char*
 librdf_uri_as_counted_string(librdf_uri *uri, size_t* len_p) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, librdf_uri, NULL);
-
+#ifdef LIBRDF_USE_RAPTOR_URI
+  return raptor_uri_as_counted_string(uri, len_p);
+#else
   if(len_p)
     *len_p=uri->string_length;
   return uri->string;
+#endif
 }
 
 
 /**
  * librdf_uri_get_digest:
+ * @world: #librdf_world object
  * @uri: #librdf_uri object
  *
  * Get a digest for the URI.
@@ -507,18 +601,21 @@ librdf_uri_as_counted_string(librdf_uri *uri, size_t* len_p)
  * Return value: new #librdf_digest object or NULL on failure.
  **/
 librdf_digest*
-librdf_uri_get_digest(librdf_uri* uri) 
+librdf_uri_get_digest(librdf_world* world, librdf_uri* uri)
 {
-  librdf_world *world=uri->world;
   librdf_digest* d;
+  unsigned char *str;
+  size_t len;
   
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, librdf_uri, NULL);
 
-  d=librdf_new_digest_from_factory(world, world->digest_factory);
+  d = librdf_new_digest_from_factory(world, world->digest_factory);
   if(!d)
     return NULL;
+
+  str = librdf_uri_as_counted_string(uri, &len);
   
-  librdf_digest_update(d, (unsigned char*)uri->string, uri->string_length);
+  librdf_digest_update(d, str, len);
   librdf_digest_final(d);
   
   return d;
@@ -538,7 +635,7 @@ librdf_uri_print (librdf_uri* uri, FILE *fh)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(uri, librdf_uri);
 
-  fputs((const char*)uri->string, fh);
+  fputs((const char*)librdf_uri_as_string(uri), fh);
 }
 
 
@@ -577,6 +674,9 @@ librdf_uri_to_string (librdf_uri* uri)
 unsigned char*
 librdf_uri_to_counted_string (librdf_uri* uri, size_t* len_p)
 {
+#ifdef LIBRDF_USE_RAPTOR_URI
+  return raptor_uri_to_counted_string(uri, len_p);
+#else
   unsigned char *s;
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, librdf_uri, NULL);
@@ -593,6 +693,7 @@ librdf_uri_to_counted_string (librdf_uri* uri, size_t* len_p)
 
   strcpy((char*)s, (const char*)uri->string);
   return s;
+#endif
 }
 
 
@@ -608,12 +709,16 @@ librdf_uri_to_counted_string (librdf_uri* uri, size_t* len_p)
 int
 librdf_uri_equals(librdf_uri* first_uri, librdf_uri* second_uri) 
 {
+#ifdef LIBRDF_USE_RAPTOR_URI
+  return raptor_uri_equals(first_uri, second_uri);
+#else
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(first_uri, librdf_uri, 0);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(second_uri, librdf_uri, 0);
 
   if(!first_uri || !second_uri)
     return 0;
   return (first_uri == second_uri);
+#endif
 }
 
 
@@ -630,7 +735,7 @@ librdf_uri_is_file_uri(librdf_uri* uri)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, librdf_uri, 1);
 
-  return raptor_uri_uri_string_is_file_uri(uri->string);
+  return raptor_uri_uri_string_is_file_uri(librdf_uri_as_string(uri));
 }
 
 
@@ -651,7 +756,7 @@ librdf_uri_to_filename(librdf_uri* uri)
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, librdf_uri, NULL);
 
-  return raptor_uri_uri_string_to_filename(uri->string);
+  return raptor_uri_uri_string_to_filename(librdf_uri_as_string(uri));
   
 }
 
@@ -670,6 +775,9 @@ librdf_uri_to_filename(librdf_uri* uri)
 int
 librdf_uri_compare(librdf_uri* uri1, librdf_uri* uri2)
 {
+#ifdef LIBRDF_USE_RAPTOR_URI
+  return raptor_uri_compare(uri1, uri2);
+#else
   if(uri1 == uri2)
     return 0;
   else if(!uri1)
@@ -678,6 +786,7 @@ librdf_uri_compare(librdf_uri* uri1, librdf_uri* uri2)
     return 1;
   else
     return strcmp((const char*)uri1->string, (const char*)uri2->string);
+#endif
 }
 
 
@@ -735,7 +844,7 @@ main(int argc, char *argv[])
 
   
   fprintf(stderr, "%s: Getting digest for URI\n", program);
-  d=librdf_uri_get_digest(uri2);
+  d = librdf_uri_get_digest(world, uri2);
   if(!d) {
     fprintf(stderr, "%s: Failed to get digest for URI %s\n", program, 
 	    librdf_uri_as_string(uri2));
