@@ -87,6 +87,19 @@ int PASSED=0;
 int FAILED=0;
 
 
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
+#define FORMAT_ATTR(string_index, first_to_check_index) \
+  __attribute__((__format__(__printf__, string_index, first_to_check_index)))
+#else
+#define FORMAT_ATTR(string_index, first_to_check_index)
+#endif
+
+
+/* make GCC happier by having 'printf' format attribute in prototype */
+static void endTest(int OK, const char *message, ...) FORMAT_ATTR(2, 3);
+static void startTest(int testID, const char *message, ...) FORMAT_ATTR(2, 3);
+
+
 static void
 endTest(int OK, const char *message, ...)
 {
@@ -130,10 +143,6 @@ getTotal(void)
 static int
 log_handler(void *user_data, librdf_log_message *message)
 {
-#ifdef RAPTOR_V2_AVAILABLE
-  librdf_world* world=(librdf_world*)user_data;
-#endif
-
   /* int code=message->code; */ /* The error code */
   raptor_locator *locator=(raptor_locator*)(message->locator);
 
@@ -147,11 +156,7 @@ log_handler(void *user_data, librdf_log_message *message)
     fprintf(stderr, ": Error - ");
 
   if(locator) { /* && message->facility == LIBRDF_FROM_PARSER) */
-#ifdef RAPTOR_V2_AVAILABLE
-    raptor_print_locator_v2(librdf_world_get_raptor(world), stderr, locator);
-#else
-    raptor_print_locator(stderr, locator);
-#endif
+    raptor_locator_print(locator, stderr);
     fputc(':', stderr);
     fputc(' ', stderr);
   }
@@ -470,7 +475,9 @@ main(int argc, char *argv[])
   /**** Test 3 *******/
   startTest(3, " Print all triples in <%s> context\n", context);
   {
-    librdf_model_print(model, stdout);
+    raptor_iostream* iostr = raptor_new_iostream_to_file_handle(librdf_world_get_raptor(world), stdout);
+    librdf_model_write(model, iostr);
+    raptor_free_iostream(iostr);
     endTest(1, "\n");
   }
 
@@ -632,7 +639,7 @@ main(int argc, char *argv[])
         count=0;
         while(!librdf_stream_end(stream)) {
           librdf_statement *stmt=librdf_stream_get_object(stream);
-          ctxt_node=(librdf_node*)librdf_stream_get_context(stream);
+          ctxt_node = librdf_stream_get_context2(stream);
           if(!stmt) {
               ok=0;
               endTest(ok, " librdf_stream_next returned NULL\n");
@@ -804,7 +811,7 @@ main(int argc, char *argv[])
         count=0;
         while(!librdf_stream_end(stream)) {
           librdf_statement *stmt=librdf_stream_get_object(stream);  /*returns SHARED pointer */
-          ctxt=(librdf_node*)librdf_stream_get_context(stream);
+          ctxt = librdf_stream_get_context2(stream);
           if(!stmt) {
               endTest(0, " librdf_stream_next returned NULL\n");
               break;
@@ -848,13 +855,18 @@ main(int argc, char *argv[])
 
         fprintf(stderr, "**: Formatting query result as '%s':\n", results_format);
 
-        iostr=raptor_new_iostream_to_file_handle(stdout);
-        formatter=librdf_new_query_results_formatter(results, results_format, NULL);
+        iostr = raptor_new_iostream_to_file_handle(librdf_world_get_raptor(world), stdout);
+        formatter = librdf_new_query_results_formatter2(results, results_format,
+                                                        NULL /* mime type */,
+                                                        NULL /* format_uri */);
 
+        base_uri = librdf_new_uri(world, (const unsigned char*)"http://example.org/");
+        
         librdf_query_results_formatter_write(iostr, formatter, results, base_uri);
-
         librdf_free_query_results_formatter(formatter);
         raptor_free_iostream(iostr);
+        librdf_free_uri(base_uri);
+
         endTest(1, "\n");
         librdf_free_query_results(results);
     }
